from ..core.sock	import download_https


def download_xml_daily(url_id, split=True):
	data = download_https('/rss/user/' + url_id, 'www.dailymotion.com')
	if data == None:
		return None
	if split:
		linfo = data.split('<item>')
		del linfo[0]
		if linfo == []:
			return None
		return linfo
	else:
		return data




import socket
import ssl
import re
import zlib
from time import time
from .tools import log

TIMEOUT = 2

def recv(sock, url):
	data = b""
	trun = time()
	lenght = None
	while True:
		raw_data = b""
		try:
			raw_data = sock.recv(100000)
		except socket.timeout:
			if lenght != None and len(data) > lenght:
				break
		except Exception as e:
			log.Warning('Error Recv: {}'.format(e))
			break
		if lenght == None and ( b'Content-Length:' in data or b'content-length:' in data ):
			headers = data.split(b'\r\n\r\n')[0].decode()
			lenght = int(re.findall(r'[Cc]ontent-[Ll]ength: (.*)\r', headers)[0]) + len(headers)
		if time() - trun > TIMEOUT:
			break
		data += raw_data
	sock.close()
	try:
		data = data.decode('utf8')
		if not re.match(r'HTTP/1\.[0-1] 200', data):
			log.Warning("Error: {} ({})".format(data.split('\r\n')[0], url))
			return None
		return data
	except:
		if b'gzip' in data:
			body = data.split(b'\r\n\r\n')[1]
			try:
				body = zlib.decompress(body, 16+zlib.MAX_WBITS)
				return body.decode()
			except:
				log.Warning('Can\'t decode data recv ({})'.format(url))
				return None

def download_http(url, host='www.youtube.com'):
	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	sock.connect((host, 80))
	sock.settimeout(0.1);
	sock.send("""GET {} HTTP/1.1
Host:{}
Accept-Language: en
Accept: */*
Accept-Encoding: gzip, raw

""".format(url.decode(), host).replace('\n', '\r\n').encode())
	return recv(sock, host + url.decode())

def download_https(url, host='www.youtube.com'):
	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	context = ssl._create_default_https_context()
	context.check_hostname = host
	sock = context.wrap_socket(sock, server_hostname=host)
	try:
		sock.connect((host, 443))
	except Exception as e:
		log.Error('{} ({}{})'.format(e, host, url))
		return None
	sock.settimeout(0.1);
	sock.write("""GET {} HTTP/1.1
Host:{}
Accept-Language: en
Accept: */*
Accept-Encoding: raw

""".format(url, host).replace('\n', '\r\n').encode())
	return recv(sock, 'https://' + host + url)



import time
import os
import sys

from .core.commands import Commands
from .core.tools import (
	log)
from .version	import __version__

def getpath():
	if os.name == 'nt':
		path = os.path.expanduser('~') + '/.youtube_sm/'
	else:
		if os.uname().sysname == 'Linux' or os.uname().sysname == 'Darwin':
			path = os.environ['HOME'] + '/.cache/youtube_sm/'
	return path

def main():
	# Init variable
	# Path of the cache
	try:
		path = getpath()
	except KeyError:
		log.Error('HOME is not set')
		exit()
	log.cache = path + 'verbose.log'
	log.Info('Hello :)')
	log.Info('Version: ', __version__)
	log.Info('Path: {}'.format(path))
	try:
		os.makedirs(path + 'data/')
	except:
		log.Warning('Data already exist or can\'t be create')
	del sys.argv[0]
	cmd = Commands(path)
	cmd.parser()
	cmd.router()



from ..core.sock	import download_https
from ..core.tools import (
	log)

import re

def download_xml_revolutionpermanente(split=True):
	data = download_https('/spip.php?page=backend', 'www.revolutionpermanente.fr')
	if data == None:
			return None
	if split:
			linfo = re.findall('<item xml:lang="fr">(.+?)</item>', data, re.DOTALL)
			if linfo == []:
					return None
			return linfo
	return data



import re
from datetime		import datetime
from threading		import Thread
from .analyzer		import Analyzer
from ..core.tools	import (
	log
)
from ..downloader.revolutionpermanente	import (
	download_xml_revolutionpermanente
)

class RevolutionPermanente_Analyzer(Thread, Analyzer):
	SITE='[revolutionpermanente]'
	URL_MATCH=r'(?:https://|)(?:www\.|)revolutionpermanente\.fr.*'

	def __init__(self, url_id='', min_date=0, mode='', method='0', file=None, prog=None):
		######################
		# The basic variable #
		######################
		self.id = url_id
		self.mode = mode
		self.method = method
		self.min_date = min_date
		###############################
		# Init the video informations #
		###############################
		self.title		= ''
		self.url		= ''
		self.url_img	= ''
		self.date		= ''
		################
		# The function #
		################
		self.prog = prog
		self.file = file

	def Thread(self):
		Thread.__init__(self)

	def run(self):
		self.analyzer_sub()
		if self.prog != None:
			self.prog.add()

	def add_sub(self, url):
		""" This function return the informations wich are write in sub.swy ."""
		return 'Revolution Permanente'

	def write(self):
		return self.file.add(
			url=self.url,
			title=self.title,
			image=self.url_img,
			uploader='Revolution Permanente',
			url_uploader='https://www.revolutionpermanente.fr/',
			date=self.date,
		)

	def info_rss(self, ele):
			self.title		= re.findall('<title>(.*)</title>', ele)[0]
			self.url		= re.findall('<link>(.*)</link>', ele)[0]
			self.url_img	= re.findall("img class='spip_logo spip_logo_right spip_logos' .* src='(.+?)'", ele)[0]
			date			= re.findall('<dc:date>(.*)</dc:date>', ele)[0]
			self.date		= datetime.strptime(date, '%Y-%m-%dT%H:%M:%SZ')

	def analyzer_sub(self):
		""" The main function wich retrieve the informations and and write it
		in a file"""
		data = download_xml_revolutionpermanente()
		if data == None:
			return
		for ele in data:
			try:
				self.info_rss(ele)
			except:
				pass
			self.write()

	def old(self, url, lcl):
		""" The function wich is call with the option -o
		This function print the old channel or the dead channel."""
		pass

	def dead(self, url):
		""" The function wich is call with the option -d
		This function print the dead channel."""
		pass

	def stat(self, sub, name):
		""" The function wich is call with the option -s
		This function print the views and the ratio of like of a video"""
		pass



import re
import time
import os
import sys

from .write import (
	Write_file,
	write_css,
	write_log)
from .thread import (
	Run_analyze,
	old,
	stats,
	dead)
from .swy import (
	swy,
	generate_swy,
	add_sub,
	add_suburl,
	init_swy)
from .tools import (
	del_data,
	log,
	exit_debug)
from .time import lcl_time
from ..version	import __version__
from ..analyzer.imports import UrlToAnalyzer

class Commands():
	def __init__(self, path):
		self.params = {
			'-a':			{'func': self._a,			'option': 'URL',		'description': "Add a sub to your sub list."},
			'-e':			{'func': self._e,			'option': '',			'description': "Edit your sub list."},
			'-h':			{'func': self._h,			'option': '',			'description': "Print this help text and exit."},
			'-l':			{'func': self._l,			'option': 'URL',		'description': "Analyze only one sub."},
			'-m':			{'func': self._m,			'option': 'MODE',		'description': "Choose the type of the output file (html, json, raw, list, view)."},
			'-r':			{'func': self._r,			'option': '',			'description': "Remove the cache."},
			'-s':			{'func': self._s,			'option': 'URL',		'description': "Stats of the selected channel(s)."},
			'-t':			{'func': self._t,			'option': 'DAYS',		'description': "Select how many DAYS ago the last content written to your file will be dated ."},
			'-v':			{'func': self.nothing,		'option': '',			'description': "Verbose."},
			'--af':			{'func': self.__af,			'option': 'FILE',		'description': "Add a list of sub to your sub list."},
			'--ax':			{'func': self.__ax,			'option': 'FILE',		'description': "Add a xml file in your sub list."},
			'--cat':		{'func': self.__cat,		'option': '',			'description': "View your subscriptions."},
			'--css':		{'func': self.__css,		'option': 'STYLE',		'description': "Export the css files (light, dark, switch)."},
			'--dead':		{'func': self.__dead,		'option': '',			'description': "Show the dead channels + those who posted no videos."},
			'--help':		{'func': self._h,			'option': '',			'description': "Print this help text and exit."},
			'--html':		{'func': self.__html,		'option': '',			'description': "Recover sub with html page instead of RSS. This method recover more video."},
			'--init':		{'func': self.__init,		'option': 'FILE',		'description': "Remove all your subs and add new."},
			'--loading':	{'func': self.__loading,	'option': '',			'description': "Print a progress bar."},
			'--old':		{'func': self.__old,		'option': 'MONTHS',		'description': "Show channels who didn\'t post videos since MONTHS + dead channels."},
			'--output':		{'func': self.__output,		'option': 'FILE',		'description': "Write the output in FILE."},
			'--ultra-html':	{'func': self.__ultra_html,	'option': '',			'description': "An advanced version of --html."},
			'--version':	{'func': self.__version,	'option': '',			'description': "Print version."},
		}
		self.url_data = []
		self.analyze = False
		self.analyze_only_one = False
		self.loading = False
		self.method = '0'
		self.mode = 'html'
		self.count = 7
		self.all_time = False
		self.output = ''
		self.path = path
		self.trun = time.time()

	def _h(self):
		print('Usage: youtube-sm [OPTIONS]')
		print()
		print('Options:')
		for name in self.params:
				param = self.params[name]
				print('  {:12} {:8}  {}'.format(name, param['option'], param['description']))
		exit()

	def _m(self, arg):
		self.analyze = True
		if arg + 1 < len(sys.argv) and sys.argv[arg + 1] in ['html', 'raw', 'list', 'view', 'json']:
			self.mode = sys.argv[arg + 1]
		else:
			exit_debug('Mode file invalid', 1)

	def _t(self, arg):
		self.analyze = True
		try:
			self.count = int(sys.argv[arg + 1])
			if self.count == -1:
				self.all_time = True
		except:
			exit_debug('Numbers of day invalid', 1)

	def _a(self, arg):
		if re.match('\[.*\]', sys.argv[arg+1]) and arg + 2 < len(sys.argv):
			add_sub({sys.argv[arg+1]: [sys.argv[arg + 2]]}, self.path)
		elif arg + 1 < len(sys.argv):
			add_suburl(sys.argv[arg+1], self.path)
		else:
			exit_debug('You Forgot An Argument (-a)', 1)

	def _e(self, arg):
		editor = os.getenv('EDITOR')
		if editor == None:
			self.RWarning('The variable `EDITOR` is not set. `vi` is use by default')
			editor = '/bin/vi'
		os.system('{} {}sub.swy'.format(editor, self.path))

	def _l(self, arg):
		self.analyze = True
		self.analyze_only_one = True
		del_data(self.path, False)
		if re.match('\[.*\]', sys.argv[arg+1]) and arg + 2 < len(sys.argv):
			self.url_data = {sys.argv[arg+1]: [sys.argv[arg+2]]}
		elif arg + 1 < len(sys.argv):
			analyzer = UrlToAnalyzer(sys.argv[arg+1])
			self.url_data = {analyzer.SITE: [sys.argv[arg+1]]}
		else:
			exit_debug('You forgot an argument after -l', 1)

	def _s(self, arg):
		try:
			if sys.argv[arg+1] == 'all':
				subs = swy(self.path, 1)
				stats(subs)
			else:
				stats({sys.argv[arg+1]: {sys.argv[arg+2]: sys.argv[arg+2]}})
		except IndexError:
			exit_debug("Missing argument after the '-s'", 1)

	def _r(self, arg):
		del_data(self.path, True)

	def __init(self, arg):
		init_swy(self.path, arg)

	def __af(self, arg):
		if arg + 1 < len(sys.argv) and os.path.exists(sys.argv[arg + 1]):
			add_sub(open(sys.argv[arg + 1], 'r').read().split('\n'), self.path)
		else:
			exit_debug('File not found', 1)

	def __ax(self, arg):
		if arg + 1 < len(sys.argv) and os.path.exists(sys.argv[arg + 1]):
			generate_swy(sys.argv[arg + 1], self.path)
		else:
			exit_debug('File not found', 1)

	def __cat(self, arg):
		if os.path.exists(self.path + 'sub.swy'):
			with open(self.path + 'sub.swy', 'r') as file:
				while True:
					line = file.readline()
					if line == '':
						break
					try:
						print(line, end='')
					except:
						print(line.encode())

	def __html(self, arg):
		self.method = '1'
		self.analyze = True

	def __old(self, arg):
		self.url_data = swy(self.path)
		self.RInfo('[*]Start of analysis')
		try:
			min_tps = int(sys.argv[arg + 1])
		except:
			old(self.url_data)
		else:
			old(self.url_data, min_tps)

	def __dead(self, arg):
		self.url_data = swy(self.path)
		self.RInfo('Start of analysis')
		dead(self.url_data)

	def __css(self, arg):
		try:
			os.mkdir('css')
		except:
			log.Error('CSS folder already exist or can\'t be created')
		if len(sys.argv) != arg + 1:
			write_css(sys.argv[arg+1])
		else:
			write_css('')

	def __ultra_html(self, arg):
		self.method = '2'
		self.analyze = True

	def __loading(self, arg):
		self.loading = True
		self.analyze = True

	def __output(self, arg):
		if arg + 1 >= len(sys.argv):
			exit_debug('You forgot an argument after --output', 1)
		self.output = sys.argv[arg+1]
		self.analyze = True

	def __version(self, arg):
		log.RInfo('Version: {}'.format(__version__))

	def __default(self):
		self.url_data = swy(self.path)
		file = Write_file('sub.html', self.path, 'html', '0', 7)
		Run_analyze(self.url_data, 'sub.html', lcl_time(), self.path, 'html', False, file, '0')
		file.write()
		write_log(sys.argv, self.path, self.trun)

	def nothing(self, arg):
		pass

	def parser(self):
		if sys.argv in [[], ['-v']]: # Default command
			self.__default()
		elif sys.argv == ['-h'] or sys.argv == ['--help']:
			self._h()
		else:
			for arg in range(len(sys.argv)):
				if len(sys.argv[arg]) == 0 or sys.argv[arg][0] != '-' or sys.argv[arg] == '-1':
					continue
				elif len(sys.argv[arg]) > 1:
					if sys.argv[arg] in self.params:
						self.params[sys.argv[arg]]['func'](arg)
					else:
						exit_debug("No such option: {}".format(sys.argv[arg]), 1)

	def router(self):
		if self.analyze:
			if not self.analyze_only_one:
				self.url_data = swy(self.path, 0)
			if self.output == '':
				if self.mode == 'html':
					self.output = 'sub.html'
				elif self.mode == 'list':
					self.output = 'sub_list'
				elif self.mode == 'raw':
					self.output = 'sub_raw'
				elif self.mode == 'json':
					self.output = 'sub.json'
			file = Write_file(self.output, self.path, self.mode, self.method, self.count)
			nb_new = Run_analyze(self.url_data, self.output, lcl_time(self.count + 30, self.all_time), self.path, self.mode, self.loading, file, self.method)
			file.write()
			write_log(sys.argv, self.path, self.trun)
		log.Info('Done ({} seconds)'.format(str(time.time() - self.trun)[:7]))



import re

from time				import strptime
from threading			import Thread
from .analyzer			import Analyzer
from ..core.tools		import log
from ..downloader.dailymotion	import download_xml_daily

class Dailymotion_Analyzer(Thread, Analyzer):
	SITE='[dailymotion]'
	URL_MATCH=r'(https://|)(www\.|)dailymotion\.com/'

	def __init__(self, url_id='', min_date=0, mode='', method='0', file=None, prog=None):
		self.id		 = url_id
		self.mode	 = mode # html / raw / list / view
		self.method	 = method # 0 --> RSS / 1 --> html / 2 --> ultra-html
		self.min_date	 = min_date
		# Init info videos
		self.url	 = "" # id of a video
		self.url_channel = ""
		self.url_img	 = ""
		self.title	 = ""
		self.channel	 = ""
		self.date	 = ""
		# Function
		self.prog = prog # True --> loading / False --> no loading
		self.file = file

	def Thread(self):
		Thread.__init__(self)

	def run(self):
		self.analyzer_sub()
		if self.prog != None:
			self.prog.add()

	def add_sub(self, sub):
		log.Warning("Dailymotion has delete the rss so this functionnality is suspende. Sorry")
		return None
		data = download_xml_daily(sub, split=False)
		if data == None:
			return None
		return sub + '\t' + sub

	def analyzer_sub(self):
		log.Warning("Dailymotion has delete the rss so this functionnality is suspende. Sorry")
		return
		if self.method == '0':
			linfo = self._download_page()
			print(linfo)
			if linfo == False or linfo == None:
				return
			for i in linfo:
				try:
					self.info_recup_rss(i)
				except:
					lor.warning('Error during the retrieve of the info ({})'.format(self.id))
				else:
					self.write()

	def _download_page(self):
		if self.method == '0':
			return download_xml_daily(self.id)
		else:
			return False

	def write(self):
		"""Write the information in a file"""
		return self.file.add(
			uploader=self.channel,
			date=self.date,
			title=self.title,
			url=self.url,
			url_uploader=self.url_channel,
			image=self.url_img,
			view=self.view,
		)

	def info_recup_rss(self, i):
		self.url = re.findall(r'<link>(.+?)</link>', i)[0]
		self.url_channel = 'https://www.dailymotion.com/' + self.id
		self.title = re.findall(r'<media:title>(.+?)</media:title>', i)[0]
		self.url_img = re.findall(r'<media:thumbnail\ url="(.+?)"', i)[0]
		self.channel = self.id
		date = re.findall(r'<pubDate>.{5}(.*)\ \+', i)[0]
		self.date = strptime(date, '%d %b %Y %H:%M:%S')
		self.view = re.findall(r'<dm:views>(.+?)</dm:views>', i)[0].replace(',', '')

	def convert_date(self, date, y=False):
		x = strptime(date, '%d %b %Y %H:%M:%S')
		month, day = str(x.tm_mon), str(x.tm_mday)
		if len(month) == 1:
			month = '0' + month
		if len(day) == 1:
			day = '0' + day
		if y:
			return str(x.tm_year) + '-' + month + '-' + day, x
		else:
			return str(x.tm_year) + '-' + month + '-' + day

	def old(self, url, lcl):
		data = download_xml_daily(url)
		if data == None:
			print('[ channel dead ]', url)
		else:
			try:
				last = int(self.convert_date(re.findall(r'<pubDate>.{5}(.*)\ \+', data[0])[0]).replace('-', ''))
			except:
				return
			if last <= lcl:
				last = str(last)
				print('[ ' + last[:4] + '/' + last[4:6] + '/' + last[6:8] + ' ] ' + url)
		return

	def dead(self, url):
		data = download_xml_daily(url)
		if data == None or data == False:
			print('[ channel dead ]', url)

	def stat(self, sub, name=''):
		view, like = 0, 0
		data = download_xml_daily(sub, False)
		views = re.findall(r'views>(.+?)</dm', data)
		likes = re.findall(r'dm:favorites>(.+?)</', data)
		for i in views:
			try:
				view += int(i)
			except:
				pass
		for i in likes:
			try:
				like += int(i)
			except:
				pass
		print(like, 'likes for', sub, '(' + str(view) + ' views)')



from ..core.tools	import log
from .dailymotion	import Dailymotion_Analyzer
from .infoconcert	import InfoConcert_Analyzer
from .peertube		import Peertube_Analyzer
from .youtube		import Youtube_Analyzer
from .revolutionpermanente import RevolutionPermanente_Analyzer
analyzers = [
	Dailymotion_Analyzer,
	InfoConcert_Analyzer,
	Peertube_Analyzer,
	Youtube_Analyzer,
	RevolutionPermanente_Analyzer,
]

def return_Analyzer(site):
	for anal in analyzers:
		if anal.SITE == site:
			return anal
	log.Error('Analyzer not found ({})'.format(site))
	return None

def UrlToAnalyzer(url):
	for anal in analyzers:
		if anal().match(url):
			return anal
	return None



import re

class Analyzer():
	def match(self, url):
		return re.match(self.URL_MATCH, url)

	def extract_id(self, url):
		if url == '':
			return ''
		match = self.match(url)
		if not match:
			return url
		return match.group('ID')
